#include <ESP8266WiFi.h>
#include "DHT.h"

const char* ssid = ""; //SSID aus dem Router
const char* password = ""; //Passwort für den Zugang zum WLAN

WiFiServer server(80);

#define DHT11PIN D4
#define DHT11TYPE DHT11
DHT dht11(DHT11PIN, DHT11TYPE);

const int MAX_DATA = 255;
float chartData[MAX_DATA][2] = {};
int counter = -1;

void setup() {
  Serial.begin(115200); //Baudrate für die Serielle Geschwindigkeit.
  delay(10); //10ms. Warten damit die Seriele Kommunikation aufgebaut wurde.

  Serial.print("Aufbau der Verbindung zu: "); //Ausgabe der SSID auf der Seriellen Schnittstelle.
  Serial.println(ssid);

  WiFi.begin(ssid, password); //Initialisieren der Wifi Verbindung.

  while (WiFi.status() != WL_CONNECTED) { //Warten bis die Verbindung aufgebaut wurde.
    delay(500);
    //Einen Punkt auf der Seriellen Schnittstelle ausgeben so das der Benutzer erkennt dass, das Sketch noch läuft.
    Serial.print(".");
  }

  //Bei erfolgreicher Verbindung wird der folgende Text ausgeben.
  Serial.print("Mit ");
  Serial.print(ssid);
  Serial.print("erfolgreich verbunden!");

  server.begin(); // Starten des Servers.
  Serial.println("Server gestartet"); //Ausgabe auf der Seriellen Schnittstelle das der Server gestartet wurde.

  // Ausgabe der IP Adresse
  Serial.print("Adresse : http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");

  dht11.begin();
}

void loop() {
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  // Wenn sich ein Client verbunden hat solange warten bis Daten gesendet werden.
  Serial.println("Neuer Client verbunden.");
  while (!client.available()) {
    delay(1);
  }

  sendRespond(client);

  client.flush();
  delay(1500);
}

void sendRespond(WiFiClient client) {
 counter++;
 if(counter > (MAX_DATA-1)){
   counter = 0;
 }

 float tempC = getTempCelsius();
 float humidity = getHumidity();
  
 chartData[counter][0] = tempC;
 chartData[counter][1] = humidity;

 String tableData = "";
  
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println("");
  client.println("<!DOCTYPE HTML>");
  client.println("<html>"); 
  client.println("<head>"); 
  client.println("<script type='text/javascript' src='https://www.gstatic.com/charts/loader.js'></script>"); 
  client.println("<script type='text/javascript' src='http://progs.draeger-it.blog/wemosd1/dht11/linechart.js'></script>"); 
  client.println("<link rel='stylesheet' href='http://progs.draeger-it.blog/wemosd1/dht11/styles.css'/>");   
  String ipAddress = WiFi.localIP().toString();
  client.println("<meta http-equiv=\"refresh\" content=\"1; URL=http://"+ ipAddress +"/\"/>");
  client.println("<title>DIY - Wetterstation - DHT11 Sensordaten</title>");
  client.println("</head>"); 
  client.println("<body>"); 
  client.println("<div id='chart' class='chartContainer' style='width: 900px; height: 500px;float:left;'></div>"); 
  client.println("<script>"); 
  client.println("google.charts.load('current', {'packages':['line']});"); 
  client.println("google.charts.setOnLoadCallback(drawCharts);"); 
  client.println("function drawCharts() {"); 

  client.println("var data = new google.visualization.DataTable();");
  client.println("data.addColumn('number', 'Zaehler');");
  client.println("data.addColumn('number', 'Temperatur in Celsius');");
  client.println("data.addColumn('number', 'rel. Luftfeuchtigkeit');");
  
  client.println(" data.addRows([");
  for(int i=0;i<=counter;i++){
    client.print("["); 
    //client.print("'"); 
    client.print(i+1); 
    //client.print("'"); 
    client.print(","); 
    client.print(chartData[i][0]); 
    client.print(","); 
    client.print(chartData[i][1]); 
    client.print("]"); 
    if(i != counter){
      client.print(",");
    }

    tableData +="<tr>";
    tableData +="<td>";
    tableData +=String(i);
    tableData +="</td>";
    tableData +="<td>";
    tableData +=String(tempC);
    tableData +="&deg;C</td>";
    tableData +="<td>";
    tableData +=String(humidity);
    tableData +="%</td>";
    tableData +="</tr>";
    
  }
  client.println("]);");
  
  client.println("drawLineChart(data);");
  client.println("}"); 
  client.println("</script>"); 
  client.println("<table border='1' style='float:left;margin-left:25px;'>"); 
  client.println("<thead>"); 
  client.println("<tr>"); 
  client.println("<th>Zaehler</th>"); 
  client.println("<th>Temperatur</th>"); 
  client.println("<th>rel. Luftfeuchtigkeit</th>"); 
  client.println("</tr>"); 
  client.println("</thead>"); 
  client.println("<tbody>"); 
  client.println(tableData);  
  client.println("</tbody>"); 
  client.println("</table>"); 
  
  client.println("</body>"); 
  client.println("</html>"); 
}

//Temperatur in Celsius vom Sensor DHT11 lesen
float getTempCelsius() {
  return getTemp(false);
}

//Temperatur in Fahrenheit vom Sensor DHT11 lesen
float getTempFahrenheit() {
  return getTemp(true);
}

//Liefert die Temperatur vom DHT11 Sensor
//der Parameter inFahrenheit steuert ob
//die Temperatur in Fahrenheit (true) oder in Celsius (false)
//geliefert wird.
float getTemp(bool inFahrenheit) {
  float tempValue = dht11.readTemperature(inFahrenheit);
  if (isnan(tempValue)) {
    Serial.println("DHT11 konnte nicht ausgelesen werden");
    tempValue = -1;
  }
  return tempValue;
}

//Lesen der relativen Luftfeuchtigkeit vom DHT11 Sensor
float getHumidity() {
  float humidity = dht11.readHumidity();
  if (isnan(humidity)) {
    Serial.println("DHT11 konnte nicht ausgelesen werden");
    humidity = -1;
  }
  return humidity;
}
